#pragma once
#include <PSX/System/IOPorts/dma_io.hpp>
#include <PSX/SPU/spu.hpp>

namespace JabyEngine {
    namespace SPU {
        namespace internal {
            namespace DMA {
                static void wait() {
                    DMA_IO::SPU.wait();
                }

                static void end() {
                    SPU_IO::ControlRegister.set_transfer_mode(SPU_IO::ControlRegister::Stop);
                }

                namespace Receive {
                    static void prepare() {
                        SPU_IO::DataTransferControl.write(SPU_IO::DataTransferControl::NormalTransferMode());
                        SPU_IO::ControlRegister.set_transfer_mode(SPU_IO::ControlRegister::Stop);
                        SPU_IO::ControlRegister.set_transfer_mode(SPU_IO::ControlRegister::DMAWrite);
                    }

                    static void set_src(uintptr_t adr) {
                        DMA_IO::SPU.set_adr(adr);
                    }

                    static void set_dst(SPU::SRAM_Adr adr) {
                        SPU_IO::SRAMTransferAdr.write(adr);
                    }

                    // Not adjusted yet
                    static void start(uint16_t blockCount, uint16_t wordsPerBlock = 0x10) {
                        using SyncMode1 = DMA_IO::BCR::SyncMode1;

                        DMA_IO::SPU.block_ctrl.write(DMA_IO::BCR::from(SyncMode1::BlockSize.with(wordsPerBlock), SyncMode1::BlockAmount.with(blockCount)));
                        DMA_IO::SPU.channel_ctrl.write(DMA_IO::CHCHR::StartSPUReceive());
                    }
                }
            }
        }
    }
}
